# TODO: Standardize / unify this code with ag.save()
import logging
import os
import pickle
import tempfile

from ..utils import compression_utils, s3_utils

logger = logging.getLogger(__name__)

compression_fn_map = compression_utils.get_compression_map()


# TODO: object -> obj?
def save(path, object, format=None, verbose=True, **kwargs):
    compression_fn = kwargs.get("compression_fn", None)
    compression_fn_kwargs = kwargs.get("compression_fn_kwargs", None)

    if compression_fn in compression_fn_map:
        validated_path = compression_utils.get_validated_path(path, compression_fn)
    else:
        raise ValueError(
            f"compression_fn={compression_fn} is not a valid compression_fn. Valid values: {compression_fn_map.keys()}"
        )

    def pickle_fn(o, buffer):
        return pickle.dump(o, buffer, protocol=4)

    save_with_fn(
        validated_path,
        object,
        pickle_fn,
        format=format,
        verbose=verbose,
        compression_fn=compression_fn,
        compression_fn_kwargs=compression_fn_kwargs,
    )


def save_with_fn(path, object, pickle_fn, format=None, verbose=True, compression_fn=None, compression_fn_kwargs=None):
    if verbose:
        logger.log(15, "Saving " + str(path))
    if s3_utils.is_s3_url(path):
        format = "s3"
    if format == "s3":
        save_s3(path, object, pickle_fn, verbose=verbose)
    else:
        path_parent = os.path.dirname(path)
        if path_parent == "":
            path_parent = "."  # Allows saving to working directory root without crashing
        os.makedirs(path_parent, exist_ok=True)

        if compression_fn_kwargs is None:
            compression_fn_kwargs = {}

        with compression_fn_map[compression_fn]["open"](path, "wb", **compression_fn_kwargs) as fout:
            pickle_fn(object, fout)


def save_s3(path: str, obj, pickle_fn, verbose=True):
    import boto3

    if verbose:
        logger.info(f"save object to {path}")
    with tempfile.TemporaryFile() as f:
        pickle_fn(obj, f)
        f.flush()
        f.seek(0)

        bucket, key = s3_utils.s3_path_to_bucket_prefix(path)
        s3_client = boto3.client("s3")
        try:
            config = boto3.s3.transfer.TransferConfig()  # enable multipart uploading for files larger than 8MB
            s3_client.upload_fileobj(f, bucket, key, Config=config)
        except Exception:
            logger.error("Failed to save object to s3")
            raise



import functools
import hashlib
import logging
import os
import sys
import uuid
import warnings
import zipfile
from typing import Optional

import boto3
import numpy as np
import requests
import tqdm

S3_PREFIX = "s3://"

INT_TYPES = (int, np.uint8, np.int8, np.int32, np.int64)
FLOAT_TYPES = (float, np.float16, np.float32, np.float64)
BOOL_TYPES = (bool, np.bool_)


if not sys.platform.startswith("win32"):
    # refer to https://github.com/untitaker/python-atomicwrites
    def replace_file(src, dst):
        """Implement atomic os.replace with linux and OSX.
        Parameters
        ----------
        src : source file path
        dst : destination file path
        """
        try:
            os.rename(src, dst)
        except OSError:
            try:
                os.remove(src)
            except OSError:
                pass
            finally:
                raise OSError(
                    "Moving downloaded temp file - {}, to {} failed. \
                    Please retry the download.".format(src, dst)
                )

else:
    import ctypes

    _MOVEFILE_REPLACE_EXISTING = 0x1
    # Setting this value guarantees that a move performed as a copy
    # and delete operation is flushed to disk before the function returns.
    # The flush occurs at the end of the copy operation.
    _MOVEFILE_WRITE_THROUGH = 0x8
    _windows_default_flags = _MOVEFILE_WRITE_THROUGH

    def _str_to_unicode(x):
        """Handle text decoding. Internal use only"""
        if not isinstance(x, str):
            return x.decode(sys.getfilesystemencoding())
        return x

    def _handle_errors(rv, src):
        """Handle WinError. Internal use only"""
        if not rv:
            msg = ctypes.FormatError(ctypes.GetLastError())
            # if the MoveFileExW fails(e.g. fail to acquire file lock), removes the tempfile
            try:
                os.remove(src)
            except OSError:
                pass
            finally:
                raise OSError(msg)

    def replace_file(src, dst):
        """Implement atomic os.replace with windows.
        refer to https://docs.microsoft.com/en-us/windows/desktop/api/winbase/nf-winbase-movefileexw
        The function fails when one of the process(copy, flush, delete) fails.
        Parameters
        ----------
        src : source file path
        dst : destination file path
        """
        _handle_errors(
            ctypes.windll.kernel32.MoveFileExW(
                _str_to_unicode(src), _str_to_unicode(dst), _windows_default_flags | _MOVEFILE_REPLACE_EXISTING
            ),
            src,
        )


def sha1sum(filename):
    """Calculate the sha1sum of a file
    Parameters
    ----------
    filename
        Name of the file
    Returns
    -------
    ret
        The sha1sum
    """
    with open(filename, mode="rb") as f:
        d = hashlib.sha1()
        for buf in iter(functools.partial(f.read, 1024 * 100), b""):
            d.update(buf)
    return d.hexdigest()


def download(
    url: str,
    path: Optional[str] = None,
    overwrite: Optional[bool] = False,
    sha1_hash: Optional[str] = None,
    retries: int = 5,
    verify_ssl: Optional[bool] = True,
) -> str:
    """Download a given URL

    Parameters
    ----------
    url
        URL to download
    path
        Destination path to store downloaded file. By default stores to the
        current directory with same name as in url.
    overwrite
        Whether to overwrite destination file if already exists.
    sha1_hash
        Expected sha1 hash in hexadecimal digits. Will ignore existing file when hash is specified
        but doesn't match.
    retries
        The number of times to attempt the download in case of failure or non 200 return codes
    verify_ssl
        Verify SSL certificates.
    Returns
    -------
    fname
        The file path of the downloaded file.
    """
    is_s3 = url.startswith(S3_PREFIX)
    if is_s3:
        s3 = boto3.resource("s3")
        if boto3.session.Session().get_credentials() is None:
            from botocore.handlers import disable_signing

            s3.meta.client.meta.events.register("choose-signer.s3.*", disable_signing)
        components = url[len(S3_PREFIX) :].split("/")
        if len(components) < 2:
            raise ValueError("Invalid S3 url. Received url={}".format(url))
        s3_bucket_name = components[0]
        s3_key = "/".join(components[1:])
    if path is None:
        fname = url.split("/")[-1]
        # Empty filenames are invalid
        assert fname, "Can't construct file-name from this URL. Please set the `path` option manually."
    else:
        path = os.path.expanduser(path)
        if os.path.isdir(path):
            fname = os.path.join(path, url.split("/")[-1])
        else:
            fname = path
    assert retries >= 0, "Number of retries should be at least 0, currently it's {}".format(retries)

    if not verify_ssl:
        warnings.warn(
            "Unverified HTTPS request is being made (verify_ssl=False). "
            "Adding certificate verification is strongly advised."
        )

    if overwrite or not os.path.exists(fname) or (sha1_hash and not sha1sum(fname) == sha1_hash):
        dirname = os.path.dirname(os.path.abspath(os.path.expanduser(fname)))
        if not os.path.exists(dirname):
            os.makedirs(dirname, exist_ok=True)
        while retries + 1 > 0:
            # Disable pyling too broad Exception
            # pylint: disable=W0703
            try:
                logging.info("Downloading {} from {}...".format(fname, url))
                if is_s3:
                    response = s3.meta.client.head_object(Bucket=s3_bucket_name, Key=s3_key)
                    total_size = int(response.get("ContentLength", 0))
                    random_uuid = str(uuid.uuid4())
                    tmp_path = "{}.{}".format(fname, random_uuid)
                    if tqdm is not None:

                        def hook(t_obj):
                            def inner(bytes_amount):
                                t_obj.update(bytes_amount)

                            return inner

                        with tqdm.tqdm(total=total_size, unit="iB", unit_scale=True) as t:
                            s3.meta.client.download_file(s3_bucket_name, s3_key, tmp_path, Callback=hook(t))
                    else:
                        s3.meta.client.download_file(s3_bucket_name, s3_key, tmp_path)
                else:
                    r = requests.get(url, stream=True, verify=verify_ssl)
                    if r.status_code != 200:
                        raise RuntimeError("Failed downloading url {}".format(url))
                    # create uuid for temporary files
                    random_uuid = str(uuid.uuid4())
                    total_size = int(r.headers.get("content-length", 0))
                    chunk_size = 1024
                    if tqdm is not None:
                        t = tqdm.tqdm(total=total_size, unit="iB", unit_scale=True)
                    with open("{}.{}".format(fname, random_uuid), "wb") as f:
                        for chunk in r.iter_content(chunk_size=chunk_size):
                            if chunk:  # filter out keep-alive new chunks
                                if tqdm is not None:
                                    t.update(len(chunk))
                                f.write(chunk)
                    if tqdm is not None:
                        t.close()
                # if the target file exists(created by other processes)
                # and have the same hash with target file
                # delete the temporary file
                if not os.path.exists(fname) or (sha1_hash and not sha1sum(fname) == sha1_hash):
                    # atomic operation in the same file system
                    replace_file("{}.{}".format(fname, random_uuid), fname)
                else:
                    try:
                        os.remove("{}.{}".format(fname, random_uuid))
                    except OSError:
                        pass
                    finally:
                        warnings.warn("File {} exists in file system so the downloaded file is deleted".format(fname))
                if sha1_hash and not sha1sum(fname) == sha1_hash:
                    raise UserWarning(
                        "File {} is downloaded but the content hash does not match."
                        " The repo may be outdated or download may be incomplete. "
                        'If the "repo_url" is overridden, consider switching to '
                        "the default repo.".format(fname)
                    )
                break
            except Exception as e:
                retries -= 1
                if retries <= 0:
                    raise e

                logging.warning(
                    "download failed due to {}, retrying, {} attempt{} left".format(
                        repr(e), retries, "s" if retries > 1 else ""
                    )
                )

    return fname


def path_expander(path, base_folder):
    path_l = path.split(";")
    return ";".join([os.path.join(base_folder, path) for path in path_l])


def protected_zip_extraction(zipfile_path, sha1_hash, folder):
    """Extract zip file to the folder.

    A signature file named ".SHA1HASH.sig" will be created if the extraction has been finished.

    Returns
    -------
    folder
        The directory to extract the zipfile
    """
    os.makedirs(folder, exist_ok=True)

    if sha1_hash:
        sha1_hash = sha1_hash[:6]
        signature = ".{}.sig".format(sha1_hash)

        if os.path.exists(os.path.join(folder, signature)):
            # We have found the signature file. Thus, we will not extract again.
            return folder
    else:
        signature = None

    # Extract the file
    logging.info("Extract files...")
    with zipfile.ZipFile(zipfile_path, "r") as zip_ref:
        zip_ref.extractall(folder)

    if signature:
        # Create the signature
        with open(os.path.join(folder, signature), "w"):
            pass

    return folder



from __future__ import annotations

import logging

import numpy as np
import pandas as pd
from sklearn.model_selection import (
    BaseCrossValidator,
    KFold,
    LeaveOneGroupOut,
    RepeatedKFold,
    RepeatedStratifiedKFold,
    StratifiedKFold,
)
from sklearn.preprocessing import KBinsDiscretizer

from .warning_filter import warning_filter

logger = logging.getLogger(__name__)


# TODO: Add binned stratification support for regression in train/val split (non CV)
class CVSplitter:
    def __init__(
        self,
        splitter_cls=None,
        n_splits: int = 5,
        n_repeats: int = 1,
        random_state: int | None = 0,
        stratify: bool = False,
        shuffle: bool = False,
        bin: bool = False,
        n_bins: int | None = None,
        groups: pd.Series = None,
    ):
        """
        Wrapper around splitter objects to perform KFold splits.
        Supports regression stratification via the `bin` and `n_bins` argument.

        Parameters
        ----------
        splitter_cls, default None
            The class to use for splitting.
            If None, will automatically be determined based off of `stratify`, `groups`, and `n_repeats`.
        n_splits : int, default 5
            The number of splits to perform.
            Ignored if `groups` is specified.
        n_repeats: int, default 1
            The number of repeated splits to perform.
            Ignored if `groups` is specified.
        random_state : int, default 0
            The seed to use when splitting the data.
        stratify : bool, default False
            If True, will stratify the splits on `y`.
        bin : bool, default False
            If True and `stratify` is True, will bin `y` into `n_bins` bins for stratification.
            Should only be used for regression and quantile tasks.
        n_bins : int, default None
            The number of bins to use when `bin` is True.
            If None, defaults to `np.floor(n_samples / n_splits)`.
        groups : pd.Series, default None
            If specified, splitter_cls will default to LeaveOneGroupOut.

        """
        self.n_splits = n_splits
        self.n_repeats = n_repeats
        self.random_state = random_state
        self.stratify = stratify
        self.shuffle = shuffle
        self.bin = bin
        self.n_bins = n_bins
        self.groups = groups
        if splitter_cls is None:
            splitter_cls = self._get_splitter_cls()
        self._splitter = self._get_splitter(splitter_cls)

    def _get_splitter_cls(self):
        if self.groups is not None:
            num_groups = len(self.groups.unique())
            if self.n_repeats != 1:
                raise AssertionError(
                    f"n_repeats must be 1 when split groups are specified. (n_repeats={self.n_repeats})"
                )
            self.n_splits = num_groups
            splitter_cls = LeaveOneGroupOut
            # pass
        elif self.stratify:
            splitter_cls = RepeatedStratifiedKFold
        else:
            splitter_cls = RepeatedKFold
        return splitter_cls

    def _get_splitter(self, splitter_cls) -> BaseCrossValidator:
        if splitter_cls == LeaveOneGroupOut:
            return splitter_cls()
        elif splitter_cls in [RepeatedKFold, RepeatedStratifiedKFold]:
            return splitter_cls(n_splits=self.n_splits, n_repeats=self.n_repeats, random_state=self.random_state)
        elif splitter_cls in [KFold, StratifiedKFold]:
            assert self.n_repeats == 1
            return splitter_cls(n_splits=self.n_splits, shuffle=self.shuffle, random_state=self.random_state)
        else:
            raise AssertionError(f"{splitter_cls} is not supported as a valid `splitter_cls` input to CVSplitter.")

    def split(self, X: pd.DataFrame | None, y: pd.Series | np.ndarray) -> list[tuple[np.ndarray, np.ndarray]]:
        if not isinstance(y, pd.Series):
            y = pd.Series(y)
        if X is None:
            X = pd.DataFrame(index=y.index)
        splitter = self._splitter
        if isinstance(splitter, (RepeatedStratifiedKFold, StratifiedKFold)):
            if self.bin:
                if self.n_bins is None:
                    n_splits = splitter.get_n_splits()
                    n_samples = len(y)

                    # ensure at least n_splits samples per bin
                    n_bins = int(np.floor(n_samples / n_splits))
                else:
                    n_bins = self.n_bins

                if n_bins > 1:
                    k_bins_discretizer = KBinsDiscretizer(
                        n_bins=n_bins, encode="ordinal", random_state=self.random_state
                    )
                    y_bin = k_bins_discretizer.fit_transform(y.to_frame())[:, 0]
                    y = pd.Series(data=y_bin, index=y.index, name=y.name)
                else:
                    if isinstance(splitter, StratifiedKFold):
                        splitter_cls = KFold
                    else:
                        splitter_cls = RepeatedKFold
                    # Don't stratify, can't bin!
                    splitter = self._get_splitter(splitter_cls=splitter_cls)

            # FIXME: There is a bug in sklearn that causes an incorrect ValueError if performing stratification and all classes have fewer than n_splits samples.
            #  This is hacked by adding a dummy class with n_splits samples, performing the kfold split, then removing the dummy samples from all resulting indices.
            #  This is very inefficient and complicated and ideally should be fixed in sklearn.
            with warning_filter():
                try:
                    out = [[train_index, test_index] for train_index, test_index in splitter.split(X, y)]
                except:
                    y_dummy = pd.concat([y, pd.Series([-1] * self.n_splits)], ignore_index=True)
                    X_dummy = pd.concat([X, X.head(self.n_splits)], ignore_index=True)
                    invalid_index = set(list(y_dummy.tail(self.n_splits).index))
                    out = [[train_index, test_index] for train_index, test_index in splitter.split(X_dummy, y_dummy)]
                    len_out = len(out)
                    for i in range(len_out):
                        train_index, test_index = out[i]
                        out[i][0] = [index for index in train_index if index not in invalid_index]
                        out[i][1] = [index for index in test_index if index not in invalid_index]
            return out
        else:
            return [[train_index, test_index] for train_index, test_index in splitter.split(X, y, groups=self.groups)]



import json
import logging
import warnings
from collections import Counter
from subprocess import PIPE, Popen


def test_check_style():
    logging.getLogger().setLevel(logging.INFO)
    logging.info("Ruff style check")

    ruff_proc = Popen(
        ["ruff", "check", ".", "--exit-zero", "--output-format", "json"],
        stdout=PIPE,
        stderr=PIPE,
    )

    stdout, stderr = ruff_proc.communicate()
    stdout_str = stdout.decode()
    stderr_str = stderr.decode()

    if stderr_str.strip():
        print("\n=== Ruff stderr ===")
        print(stderr_str)

    try:
        diagnostics = json.loads(stdout_str) if stdout_str.strip() else []
    except json.JSONDecodeError as e:
        print("\n=== Ruff stdout (failed to parse as JSON) ===")
        print(stdout_str)
        raise AssertionError(f"Failed to parse Ruff JSON output: {e}") from e

    total_count = len(diagnostics)

    if total_count > 0:
        print("\n=== Ruff warnings ===")
        per_file_counts = Counter()

        for d in diagnostics:
            file_path = d.get("filename", "<unknown>")
            per_file_counts[file_path] += 1

            loc = d.get("location", {}) or {}
            row = loc.get("row", "?")
            col = loc.get("column", "?")

            code = d.get("code", "")
            message = d.get("message", "")
            print(f"{file_path}:{row}:{col}: {code} {message}")

        print("\n=== Ruff warnings per file ===")
        for file_path, count in per_file_counts.most_common():
            print(f"{file_path}: {count}")

        warnings.warn(f"{total_count} Ruff warnings remaining")

    assert total_count < 100, "Too many Ruff warnings found, improve code quality to pass test."



from __future__ import annotations

from pathlib import Path
from typing import Any, Mapping
from urllib.parse import urlparse


def _is_s3_uri(path: str) -> bool:
    return urlparse(path).scheme == "s3"


def _parse_s3_uri(uri: str) -> tuple[str, str]:
    p = urlparse(uri)
    if p.scheme != "s3":
        raise ValueError(f"Not an s3 uri: {uri!r}")
    bucket = p.netloc
    key = p.path.lstrip("/")
    if not bucket or not key:
        raise ValueError(f"Invalid s3 uri (expected s3://bucket/key): {uri!r}")
    return bucket, key


def _s3_put_text(
    uri: str,
    text: str,
    *,
    content_type: str = "text/yaml; charset=utf-8",
    upload_as_public: bool = False,
) -> None:
    import boto3

    bucket, key = _parse_s3_uri(uri)

    put_kwargs = dict(
        Bucket=bucket,
        Key=key,
        Body=text.encode("utf-8"),
        ContentType=content_type,
    )

    if upload_as_public:
        put_kwargs["ACL"] = "public-read"

    boto3.client("s3").put_object(**put_kwargs)


def presets_to_yaml_files(
    presets: Mapping[str, Mapping[str, Any]] | Mapping[str, Any],
    output: str | Path,
    *,
    multi: bool | None = None,
    per_preset_files: bool = False,
    overwrite: bool = False,
    sort_keys: bool = False,
    upload_as_public: bool = False,  # NEW
) -> list[str]:
    """
    Export AutoGluon-style presets to YAML.

    Supports:
      - Local file output (single file) or local directory output (per preset files)
      - S3 output:
          * single file:  s3://bucket/path/presets.yaml
          * per preset:   s3://bucket/path/presets/   (prefix; each becomes <name>.yaml)

    Notes
    -----
    - overwrite=False cannot be reliably enforced on S3 without an extra HEAD call; this implementation
      will enforce overwrite for local paths, and will *best-effort* enforce it for S3 via head_object.
    - Returns list of destinations written (local paths or s3 uris as strings).
    """
    output_str = str(output)

    def _infer_is_multi(obj: Mapping[str, Any]) -> bool:
        if not obj:
            return False
        return all(isinstance(v, Mapping) for v in obj.values())

    if multi is None:
        multi = _infer_is_multi(presets)  # type: ignore[arg-type]

    if multi:
        if not isinstance(presets, Mapping) or (presets and not all(isinstance(v, Mapping) for v in presets.values())):
            raise TypeError("multi=True expects named presets like {'best_quality': {...}, 'fast': {...}}")
        named_presets: dict[str, dict[str, Any]] = {str(k): dict(v) for k, v in presets.items()}  # type: ignore[assignment]
    else:
        single_preset: dict[str, Any] = dict(presets)  # type: ignore[arg-type]

    def _dump_yaml_text(obj: Any) -> str:
        import yaml

        return yaml.safe_dump(
            obj,
            default_flow_style=False,
            sort_keys=sort_keys,
            allow_unicode=True,
        )

    def _local_write_text(path: Path, text: str) -> None:
        if path.exists() and not overwrite:
            raise FileExistsError(f"Refusing to overwrite existing file: {path}")
        path.parent.mkdir(parents=True, exist_ok=True)
        path.write_text(text, encoding="utf-8")

    def _s3_exists(uri: str) -> bool:
        # best-effort exists check
        import boto3
        from botocore.exceptions import ClientError

        bucket, key = _parse_s3_uri(uri)
        try:
            boto3.client("s3").head_object(Bucket=bucket, Key=key)
            return True
        except ClientError as e:
            code = e.response.get("Error", {}).get("Code", "")
            if code in ("404", "NoSuchKey", "NotFound"):
                return False
            # If access is denied, we can't determine existence; treat as "unknown"
            if code in ("403", "AccessDenied"):
                return False
            raise

    written: list[str] = []

    # ---- per-preset output ----
    if per_preset_files:
        if not multi:
            raise ValueError("per_preset_files=True requires multi=True / named presets input.")

        if _is_s3_uri(output_str):
            # Interpret output as a prefix (ensure it ends with '/')
            prefix_uri = output_str if output_str.endswith("/") else (output_str + "/")
            for name, preset in named_presets.items():
                uri = f"{prefix_uri}{name}.yaml"
                if not overwrite and _s3_exists(uri):
                    raise FileExistsError(f"Refusing to overwrite existing S3 object: {uri}")
                _s3_put_text(uri, _dump_yaml_text(preset), upload_as_public=upload_as_public)
                written.append(uri)
            return written
        else:
            out_dir = Path(output_str)
            out_dir.mkdir(parents=True, exist_ok=True)
            for name, preset in named_presets.items():
                path = out_dir / f"{name}.yaml"
                _local_write_text(path, _dump_yaml_text(preset))
                written.append(str(path))
            return written

    # ---- single-file output ----
    if _is_s3_uri(output_str):
        # output must be a concrete key ending with .yml/.yaml
        if not output_str.lower().endswith((".yaml", ".yml")):
            raise ValueError(f"S3 output must be a .yaml/.yml object key: {output_str!r}")

        if not overwrite and _s3_exists(output_str):
            raise FileExistsError(f"Refusing to overwrite existing S3 object: {output_str}")

        obj = named_presets if multi else single_preset
        _s3_put_text(output_str, _dump_yaml_text(obj), upload_as_public=upload_as_public)
        written.append(output_str)
        return written

    # local single file
    out_path = Path(output_str)
    if out_path.suffix.lower() not in {".yaml", ".yml"}:
        raise ValueError(f"output must be a .yaml/.yml file when per_preset_files=False: {out_path}")
    obj = named_presets if multi else single_preset
    _local_write_text(out_path, _dump_yaml_text(obj))
    written.append(str(out_path))
    return written



#!/usr/bin/env python
###########################
# This code block is a HACK (!), but is necessary to avoid code duplication. Do NOT alter these lines.
import importlib.util
import os

from setuptools import setup

filepath = os.path.abspath(os.path.dirname(__file__))
filepath_import = os.path.join(filepath, "..", "core", "src", "autogluon", "core", "_setup_utils.py")
if not os.path.exists(filepath_import):
    filepath_import = os.path.join(filepath, "_setup_utils.py")

spec = importlib.util.spec_from_file_location("ag_min_dependencies", filepath_import)
ag = importlib.util.module_from_spec(spec)  # type: ignore
# Identical to `from autogluon.core import _setup_utils as ag`, but works without `autogluon.core` being installed.
spec.loader.exec_module(ag)  # type: ignore
###########################

version = ag.load_version_file()
version = ag.update_version(version, use_file_if_exists=False, create_file=True)

submodule = "common"
install_requires = (
    [
        # version ranges added in ag.get_dependency_version_ranges()
        "numpy",  # version range defined in `core/_setup_utils.py`
        "pandas",  # version range defined in `core/_setup_utils.py`
        "pyarrow",  # version range defined in `core/_setup_utils.py`
        "boto3",  # version range defined in `core/_setup_utils.py`
        "psutil",  # version range defined in `core/_setup_utils.py`
        "tqdm",  # version range defined in `core/_setup_utils.py`
        "requests",
        "joblib",  # version range defined in `core/_setup_utils.py`
        "pyyaml",  # version range defined in `core/_setup_utils.py`
        # s3fs is removed due to doubling install time due to version range resolution
        # "s3fs",  # version range defined in `core/_setup_utils.py`
    ]
    if not ag.LITE_MODE
    else {
        "numpy",  # version range defined in `core/_setup_utils.py`
        "pandas",  # version range defined in `core/_setup_utils.py`
        "tqdm",  # version range defined in `core/_setup_utils.py`
        "requests",
    }
)

extras_require = dict()

test_requirements = [
    "pytest",
    "types-requests",
    "types-setuptools",
    "pytest-mypy",
]

test_requirements = list(set(test_requirements))
extras_require["tests"] = test_requirements

install_requires = ag.get_dependency_version_ranges(install_requires)
for key in extras_require:
    extras_require[key] = ag.get_dependency_version_ranges(extras_require[key])

if __name__ == "__main__":
    ag.create_version_file(version=version, submodule=submodule)
    setup_args = ag.default_setup_args(version=version, submodule=submodule)
    setup(
        install_requires=install_requires,
        extras_require=extras_require,
        **setup_args,
    )



def _gzip_open(*args, **kwargs):
    import gzip

    return gzip.open(*args, **kwargs)


def _bz2_open(*args, **kwargs):
    import bz2

    return bz2.open(*args, **kwargs)


# Lazy import to avoid the following issue if users installed via pyenv:
#  https://stackoverflow.com/questions/57743230/userwarning-could-not-import-the-lzma-module-your-installed-python-is-incomple
def _lzma_open(*args, **kwargs):
    import lzma

    return lzma.open(*args, **kwargs)


compression_fn_map = {
    None: {
        "open": open,
        "extension": "",
    },
    "gzip": {
        "open": _gzip_open,
        "extension": "gz",
    },
    "bz2": {
        "open": _bz2_open,
        "extension": "bz2",
    },
    "lzma": {
        "open": _lzma_open,
        "extension": "lzma",
    },
}


def get_validated_path(filename, compression_fn=None):
    if compression_fn is not None:
        validated_path = f"{filename}.{compression_fn_map[compression_fn]['extension']}"
    else:
        validated_path = filename
    return validated_path


def get_compression_map():
    return compression_fn_map
